@@ -1,5 +1,5 @@
 #
-# $Id: mailto.pm,v 1.10 2003/10/14 18:04:09 gisle Exp $
+# $Id: mailto.pm,v 1.11 2003/10/23 19:11:33 uid39246 Exp $
 #
 # This module implements the mailto protocol.  It is just a simple
 # frontend to the Unix sendmail program except on MacOS, where it uses
@@ -75,7 +75,8 @@ sub request
 	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
 	               "You don't have SMTPHOSTS defined";
 	}
-    } else {
+    }
+    else {
 	unless (-x $SENDMAIL) {
 	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
 	               "You don't have $SENDMAIL";
@@ -85,14 +86,16 @@ sub request
 	    $mail = Mail::Internet->new or
 	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
 	    "Can't get a Mail::Internet object";
-    } else {
+    }
+    else {
 	open(SENDMAIL, "| $SENDMAIL -oi -t") or
 	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
 	               "Can't run $SENDMAIL: $!";
     }
     if ($^O eq "MacOS") {
 	$addr = $url->encoded822addr;
-    } else {
+    }
+    else {
 	$request = $request->clone;  # we modify a copy
 	my @h = $url->headers;  # URL headers override those in the request
 	while (@h) {
@@ -101,7 +104,8 @@ sub request
 	    next unless defined $v;
 	    if (lc($k) eq "body") {
 		$request->content($v);
-	    } else {
+	    }
+	    else {
 		$request->push_header($k => $v);
 	    }
 	}
@@ -109,7 +113,8 @@ sub request
     if ($^O eq "MacOS") {
 	$mail->add(To => $addr);
 	$mail->add(split(/[:\n]/,$request->headers_as_string));
-    } else {
+    }
+    else {
 	print SENDMAIL $request->headers_as_string;
 	print SENDMAIL "\n";
     }
@@ -122,11 +127,13 @@ sub request
 		foreach (@text) {
 		    $_ .= "\n";
 		}
-	    } else {
+	    }
+	    else {
 	    print SENDMAIL $$contRef;
 	    }
 
-	} elsif (ref($contRef) eq 'CODE') {
+	}
+	elsif (ref($contRef) eq 'CODE') {
 	    # Callback provides data
 	    my $d;
 	    if ($^O eq "MacOS") {
@@ -138,7 +145,8 @@ sub request
 		foreach (@text) {
 		    $_ .= "\n";
 		}
-	    } else {
+	    }
+	    else {
 		print SENDMAIL $d;
 	    }
 	}
@@ -149,7 +157,8 @@ sub request
 	    return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
 				       "Mail::Internet->smtpsend unable to send message to <$addr>");
 	}
-    } else {
+    }
+    else {
 	unless (close(SENDMAIL)) {
 	    my $err = $! ? "$!" : "Exit status $?";
 	    return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
@@ -164,7 +173,8 @@ sub request
     if ($^O eq "MacOS") {
 	$response->header('Server' => "Mail::Internet $Mail::Internet::VERSION");
 	$response->content("Message sent to <$addr>\n");
-    } else {
+    }
+    else {
 	$response->header('Server' => $SENDMAIL);
 	my $to = $request->header("To");
 	$response->content("Message sent to <$to>\n");
